# -*- coding: utf-8 -*

import getopt
import os
import sys

from east import applications
from east.synonyms import synonyms
from east import utils


def main():
    args = sys.argv[1:]
    opts, args = getopt.getopt(args, "a:l:t:sd")
    opts = dict(opts)
    opts.setdefault("-a", "easa")
    opts.setdefault("-l", "0.6")
    opts.setdefault("-t", "0.25")

    if len(args) < 2:
        print "Invalid syntax."
        return 1

    command_group = args[0]
    command = args[1]

    if command_group == "keyphrases":

        keyphrases_file = os.path.abspath(args[2])
        input_path = os.path.abspath(args[3])
        use_synonyms = "-s" in opts
        normalized_scores = "-d" not in opts
        ast_algorithm = opts["-a"]
        significance_level = float(opts["-l"])
        score_threshold = float(opts["-t"])

        if os.path.isdir(input_path):
            input_files = [os.path.abspath(input_path) + "/" + filename
                           for filename in os.listdir(input_path)
                           if filename.endswith(".txt")]
        else:
            input_files = [os.path.abspath(input_path)]

        texts = {}
        for filename in input_files:
            with open(filename) as f:
                text_name = os.path.basename(filename).decode("utf-8")[:-4]
                texts[text_name] = f.read()

        with open(keyphrases_file) as f:
            keyphrases = map(lambda k: utils.prepare_text(k), f.read().splitlines())

        if use_synonyms:
            synonimizer = synonyms.SynonymExtractor(input_path)
        else:
            synonimizer = None

        if command == "table":

            keyphrases_table = applications.keyphrases_table(keyphrases, texts, ast_algorithm,
                                                             normalized_scores, synonimizer)
            res = u""
            for keyphrase in sorted(keyphrases_table.keys()):
                res += u'<keyphrase value="%s">\n' % keyphrase
                for text in sorted(keyphrases_table[keyphrase].keys()):
                    res += u'  <text name="%s">' % text
                    res += u'%.3f' % keyphrases_table[keyphrase][text]
                    res += u'</text>\n'
                res += u'</keyphrase>\n'

            print res.encode("utf-8", "ignore")

        elif command == "graph":

            graph = applications.keyphrases_graph(keyphrases, texts, significance_level,
                                                  score_threshold, ast_algorithm,
                                                  normalized_scores, synonimizer)
            res = u""
            for node in graph:
                if graph[node]:
                    res += "%s -> %s\n" % (node, ", ".join(graph[node]))

            print res.encode("utf-8", "ignore")

        else:
            print "Invalid command."
            return 1

    else:
        print "Invalid command."
        return 1



if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*

import platform

from east import consts


def determine_operating_system():
    system = platform.system()
    bits = platform.architecture()[0]
    if system == "Windows":
        return consts.OperatingSystem.Windows
    elif system == "Linux":
        if bits.startswith("64"):
            return consts.OperatingSystem.LINUX_64
        else:
            return consts.OperatingSystem.LINUX_32
    else:
        return consts.OperatingSystem.MACOS



# -*- coding: utf-8 -*

import itertools
import numpy as np

from east.asts import base
from east.asts import utils
from east import utils as common_utils

class EnhancedAnnotatedSuffixArray(base.AST):

    __algorithm__ = "easa"

    def __init__(self, strings_collection):
        super(EnhancedAnnotatedSuffixArray, self).__init__(strings_collection)
        self.strings_collection = strings_collection
        self.string = "".join(utils.make_unique_endings(strings_collection))
        self.suftab = self._compute_suftab(self.string)
        self.lcptab = self._compute_lcptab(self.string, self.suftab)
        self.childtab_up, self.childtab_down = self._compute_childtab(self.lcptab)
        self.childtab_next_l_index = self._compute_childtab_next_l_index(self.lcptab)
        self.anntab = self._compute_anntab(self.suftab, self.lcptab)

    def score(self, query, normalized=True, synonimizer=None):
        if synonimizer:
            synonyms = synonimizer.get_synonyms()
            query_words = common_utils.tokenize(query)
            for i in xrange(len(query_words)):
                query_words[i] = synonyms[query_words[i]] + [query_words[i]]
            possible_queries = map(lambda words: "".join(words),
                                   itertools.product(*query_words))
            return max(self._score(q) for q in possible_queries)
        else:
            return self._score(query.replace(" ", ""), normalized)

    def traverse_depth_first_pre_order(self, callback):
        """Visits the internal "nodes" of the enhanced suffix array in depth-first pre-order.

        Based on Abouelhoda et al. (2004).
        """
        n = len(self.suftab)
        root = [0, 0, n - 1, ""]  # <l, i, j, char>

        def _traverse_top_down(interval):  # TODO: Rewrite with stack? As in bottom-up
            callback(interval)
            i, j = interval[1], interval[2]
            if i != j:
                children = self._get_child_intervals(i, j)
                children.sort(key=lambda child: child[3])
                for child in children:
                    _traverse_top_down(child)

        _traverse_top_down(root)

    def traverse_depth_first_post_order(self, callback):
        """Visits the internal "nodes" of the enhanced suffix array in depth-first post-order.

        Kasai et. al. (2001), Abouelhoda et al. (2004).
        """
        # a. Reimplement without python lists?..
        # b. Interface will require it to have not internal nodes only?..
        #    (but actually this implementation gives a ~2x gain of performance)
        last_interval = None
        n = len(self.suftab)
        stack = [[0, 0, None, []]]  # <l, i, j, children>
        for i in xrange(1, n):
            lb = i - 1
            while self.lcptab[i] < stack[-1][0]:
                stack[-1][2] = i - 1
                last_interval = stack.pop()
                callback(last_interval)
                lb = last_interval[1]
                if self.lcptab[i] <= stack[-1][0]:
                    stack[-1][3].append(last_interval)
                    last_interval = None
            if self.lcptab[i] > stack[-1][0]:
                if last_interval:
                    stack.append([self.lcptab[i], lb, None, [last_interval]])
                    last_interval = None
                else:
                    stack.append([self.lcptab[i], lb, None, []])
        stack[-1][2] = n - 1
        callback(stack[-1])

    def traverse_breadth_first(self, callback):
        """Visits the internal "nodes" of the enhanced suffix array in breadth-first order."""
        raise NotImplementedError

    def _score(self, query, normalized=True):
        query = query.upper()
        result = 0
        n = len(self.suftab)

        root_interval = (0, 0, n - 1)
    
        for suffix_start in xrange(len(query)):
            
            suffix = query[suffix_start:]
            suffix_score = 0
            matched_chars = 0
            nodes_matched = 0
            
            parent_node = root_interval
            child_node = self._get_child_interval(parent_node[1], parent_node[2], suffix[0])
            while child_node:
                nodes_matched += 1
                # TODO: Use structs??? child_node[1] is actually cn.i; parent_node[0] == pn.l
                substr_start = self.suftab[child_node[1]] + parent_node[0]
                if self._is_leaf(child_node):
                    substr_end = n
                else:
                    substr_end = substr_start + child_node[0] - parent_node[0]
                match = utils.match_strings(suffix, self.string[substr_start:substr_end])
                suffix_score += float(self._annotation(child_node)) / self._annotation(parent_node)
                matched_chars += match
                suffix = suffix[match:]
                if suffix and match == substr_end - substr_start:
                    parent_node = child_node
                    child_node = self._get_child_interval(parent_node[1], parent_node[2], suffix[0])
                else:
                    break

            if matched_chars:
                if normalized:
                    suffix_result = (suffix_score + matched_chars - nodes_matched) / matched_chars
                else:
                    suffix_result = (suffix_score + matched_chars - nodes_matched)
                result += suffix_result
            
        result /= len(query)
        return result

    def _compute_suftab(self, string):
        """Computes the suffix array of a string in O(n).

        The code is based on that from the pysuffix library (https://code.google.com/p/pysuffix/).

        Kärkkäinen & Sanders (2003).
        """
        n = len(string)
        string += (unichr(1) * 3)
        suftab = np.zeros(n, dtype=np.int)
        alpha = sorted(set(string))
        self._kark_sort(string, suftab, n, alpha)
        return suftab

    def _kark_sort(self, s, SA, n, alpha):
        n0 = (n + 2) / 3
        n1 = (n + 1) / 3
        n2 = n / 3
        n02 = n0 + n2

        SA12 = [0] * (n02 + 3)
        SA0 = [0] * n0
        s12 = [i for i in xrange(n + n0 - n1) if i % 3 != 0] + [0, 0, 0]

        self._radixpass(s12, SA12, s[2:], n02, alpha)
        self._radixpass(SA12, s12, s[1:], n02, alpha)
        self._radixpass(s12, SA12, s, n02, alpha)
  
        name = 0
        c0, c1, c2 = -1, -1, -1
        array_name = [0]
        for i in xrange(n02):
            if s[SA12[i]] != c0 or s[SA12[i] + 1] != c1 or s[SA12[i] + 2] != c2:
                name += 1
                array_name.append(name)
                c0 = s[SA12[i]]
                c1 = s[SA12[i]+1]
                c2 = s[SA12[i]+2]
            if SA12[i] % 3 == 1:
                s12[SA12[i] / 3] = name
            else:
                s12[SA12[i] / 3 + n0] = name

        if name < n02:
            self._kark_sort(s12, SA12, n02, array_name)
            for i in xrange(n02): 
                s12[SA12[i]] = i+1
        else:
            for i in xrange(n02): 
                SA12[s12[i]-1] = i

        s0 = [SA12[i] * 3 for i in xrange(n02) if SA12[i] < n0]

        self._radixpass(s0, SA0, s, n0, alpha)
  
        p = j = k = 0
        t = n0 - n1
        while k < n:
            i = SA12[t] * 3 + 1 if SA12[t] < n0 else (SA12[t] - n0) * 3 + 2
            j = SA0[p] if p < n0 else 0
 
            if SA12[t] < n0:
                test = (s12[SA12[t]+n0] <= s12[j/3]) if(s[i]==s[j]) else (s[i] < s[j])
            elif(s[i]==s[j]) :
                test = s12[SA12[t]-n0+1] <= s12[j/3 + n0] if(s[i+1]==s[j+1]) else s[i+1] < s[j+1]
            else:
                test = s[i] < s[j]

            if test:
                SA[k] = i
                t += 1
                if t == n02: 
                    k += 1
                    l = n0 - p
                    while p < n0:
                        SA[k] = SA0[p]
                        p += 1
                        k += 1          
            else: 
                SA[k] = j
                p += 1
                if p == n0:
                    k += 1
                    while t < n02:
                        SA[k] = (SA12[t] * 3) + 1 if SA12[t] < n0 else ((SA12[t] - n0) * 3) + 2
                        t += 1
                        k += 1
            k += 1

    def _radixpass(self, a, b, r, n, alpha):
        c = {}
        for letter in alpha:
            c[letter] = 0
        for i in xrange(n):
            c[r[a[i]]] += 1

        total = 0
        for letter in alpha:
            freq, c[letter] = c[letter], total
            total += freq
        for i in xrange(n):
            b[c[r[a[i]]]] = a[i]
            c[r[a[i]]] += 1

        return b

    def _compute_lcptab(self, string, suftab):
        """Computes the LCP array in O(n) based on the input string & its suffix array.

        Kasai et al. (2001).
        """
        n = len(suftab)
        rank = [0] * n
        for i in xrange(n):
            rank[suftab[i]] = i
        lcptab = np.zeros(n, dtype=np.int)
        h = 0
        for i in xrange(n):
            if rank[i] >= 1:
                j = suftab[rank[i] - 1]
                while string[i + h] == string[j + h]:
                    h += 1
                lcptab[rank[i]] = h
                if h > 0:
                    h -= 1
        return lcptab

    def _compute_childtab(self, lcptab):
        """Computes the child 'up' and 'down' arrays in O(n) based on the LCP table.

        Abouelhoda et al. (2004).
        """
        last_index = -1
        stack = [0]
        n = len(lcptab)
        childtab_up = np.zeros(n, dtype=np.int)  # Zeros / -1 ?
        childtab_down = np.zeros(n, dtype=np.int)
        for i in xrange(n):
            while lcptab[i] < lcptab[stack[-1]]:
                last_index = stack.pop()
                if lcptab[i] <= lcptab[stack[-1]] and lcptab[stack[-1]] != lcptab[last_index]:
                    childtab_down[stack[-1]] = last_index
            if last_index != -1:
                childtab_up[i] = last_index
                last_index = -1
            stack.append(i)
        return childtab_up, childtab_down

    def _compute_childtab_next_l_index(self, lcptab):
        """Computes the child 'next l index' array in O(n) based on the LCP table.

        Abouelhoda et al. (2004).
        """
        stack = [0]
        n = len(lcptab)
        childtab_next_l_index = np.zeros(n, dtype=np.int)  # Zeros / -1 ?
        for i in xrange(n):
            while lcptab[i] < lcptab[stack[-1]]:
                stack.pop()
            if lcptab[i] == lcptab[stack[-1]]:
                last_index = stack.pop()
                childtab_next_l_index[last_index] = i
            stack.append(i)
        return childtab_next_l_index

    def _compute_anntab(self, suftab, lcptab):
        """Computes the annotations array in O(n) by "traversing" the suffix array.

        Based on ideas from Abouelhoda et al. (2004) and Dubov & Chernyak (2013).
        """
        n = len(suftab)
        anntab = np.zeros(n, dtype=np.int)  # Zeros / -1 ?

        def process_node(node):
            # NOTE(msdubov): Assumes that child l-[i..j] lcp intervals come in the ascending
            #                order (by i). This allows to handle the leafs properly.
            i = node[1]
            for child_node in node[3]:
                if i < child_node[1]:
                    anntab[self._interval_index(node)] += child_node[1] - i
                anntab[self._interval_index(node)] += anntab[self._interval_index(child_node)]
                i = child_node[2] + 1
            if i <= node[2]:
                anntab[self._interval_index(node)] += node[2] - i + 1

        self.traverse_depth_first_post_order(process_node)
        # NOTE(msdubov): Removing the "degenerate" 1st-level leafs
        #                with the auxiliary symbol in the arc.
        anntab[0] -= len(self.strings_collection)

        return anntab

    def _interval_index(self, lcp_interval):
        """Maps an lcp interval to an index in [0..n-1].

        :param lcp_interval: <l, i, j>.
        """
        return utils.index(self.lcptab, lcp_interval[0], lcp_interval[1])

    def _annotation(self, lcp_interval):
        if self._is_leaf(lcp_interval):
            return 1
        else:
            return self.anntab[self._interval_index(lcp_interval)]

    def _is_leaf(self, lcp_interval):
        return lcp_interval[1] == lcp_interval[2]

    def _lcp_value(self, i, j):
        n = len(self.suftab)
        if (i == 0 or i == n - 1) and j == n - 1:
            return 0  # TODO: Verify the correctness of this step.
        elif i < self.childtab_up[j + 1] <= j:
            return self.lcptab[self.childtab_up[j + 1]]
        else:
            return self.lcptab[self.childtab_down[i]]

    def _get_child_intervals(self, i, j):
        if i == j:
            return []
        n = len(self.suftab)
        l = self._lcp_value(i, j)
        intervals = []
        if i == 0 and j == n - 1:
            i1 = 0  # TODO: Verify the correctness of this step.
        else:
            if i < self.childtab_up[j + 1]:
                i1 = self.childtab_up[j + 1]
            else:
                i1 = self.childtab_down[i]
            intervals.append((self._lcp_value(i, i1 - 1), i, i1 - 1, self.string[self.suftab[i] + l]))
        while self.childtab_next_l_index[i1] != 0:
            i2 = self.childtab_next_l_index[i1]
            intervals.append((self._lcp_value(i1, i2 - 1), i1, i2 - 1, self.string[self.suftab[i1] + l]))
            i1 = i2
        intervals.append((self._lcp_value(i1, j), i1, j, self.string[self.suftab[i1] + l]))
        return intervals

    def _get_child_interval(self, i, j, char):
        if i == j:
            return None
        n = len(self.suftab)
        l = self._lcp_value(i, j)
        if i == 0 and j == n - 1:
            i1 = 0
        else:
            if i < self.childtab_up[j + 1]:
                i1 = self.childtab_up[j + 1]
            else:
                i1 = self.childtab_down[i]
            if self.string[self.suftab[i] + l] == char:
                return (self._lcp_value(i, i1 - 1), i, i1 - 1, self.string[self.suftab[i] + l])
        while self.childtab_next_l_index[i1] != 0:
            i2 = self.childtab_next_l_index[i1]
            if self.string[self.suftab[i1] + l] == char:
                return (self._lcp_value(i1, i2 - 1), i1, i2 - 1, self.string[self.suftab[i1] + l])
            i1 = i2
        if self.string[self.suftab[i1] + l] == char:
            return (self._lcp_value(i1, j), i1, j, self.string[self.suftab[i1] + l])
        return None



# -*- coding: utf-8 -*

import abc
import inspect

from east import consts
from east import exceptions
from east import utils

class AST(object):
    __metaclass__ = abc.ABCMeta

    @staticmethod
    def get_ast(ast_algorithm, strings_collection):
        for ast_cls in utils.itersubclasses(AST):
            if not inspect.isabstract(ast_cls) and ast_algorithm == ast_cls.__algorithm__:
                return ast_cls(strings_collection)
        raise exceptions.NoSuchASTAlgorithm(name=ast_algorithm)

    def __init__(self, strings_collection):
        if not strings_collection:
            raise exceptions.EmptyStringsCollectionException()

    @abc.abstractmethod
    def score(self, query, normalized=True, synonimizer=None):
        """Computes the matching score for the given string against the AST."""

    def traverse(self, callback, order=consts.TraversalOrder.DEPTH_FIRST_PRE_ORDER):        
        if order == consts.TraversalOrder.DEPTH_FIRST_PRE_ORDER:
            self.traverse_depth_first_pre_order(callback)
        elif order == consts.TraversalOrder.DEPTH_FIRST_POST_ORDER:
            self.traverse_depth_first_post_order(callback)
        elif order == consts.TraversalOrder.BREADTH_FIRST:
            self.traverse_breadth_first(callback)

    @abc.abstractmethod
    def traverse_depth_first_pre_order(self, callback):
        """Traverses the annotated suffix tree in depth-first pre-order."""

    @abc.abstractmethod
    def traverse_depth_first_post_order(self, callback):
        """Traverses the annotated suffix tree in depth-first post-order."""

    @abc.abstractmethod
    def traverse_breadth_first(self, callback):
        """Traverses the annotated suffix tree in breadth-first order."""



# -*- coding: utf-8 -*

import testtools
import unittest

from east import utils


class UtilsTestCase(testtools.TestCase):

    def test_tokenize(self):
        text = "Well, what a sunny day!"
        tokens = ["Well", "what", "a", "sunny", "day"]
        self.assertEqual(utils.tokenize(text), tokens)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*

from east import utils


class _TraversalOrder(utils.ImmutableMixin, utils.EnumMixin):
    DEPTH_FIRST_PRE_ORDER = "depth-first|pre-order"
    DEPTH_FIRST_POST_ORDER = "depth-first|post-order"
    BREADTH_FIRST = "breadth-first"


class _OperatingSystem(utils.ImmutableMixin, utils.EnumMixin):
    LINUX_32 = "linux_32"
    LINUX_64 = "linux_64"
    WINDOWS = "windows"
    MACOS = "macos"


class _URL(utils.ImmutableMixin, utils.EnumMixin):
    TOMITA = "http://api.yandex.ru/tomita/download.xml"


class _String(utils.ImmutableMixin, utils.EnumMixin):
    UNICODE_SPECIAL_SYMBOLS_START = 0x0A00


TraversalOrder = _TraversalOrder()
OperatingSystem = _OperatingSystem()
URL = _URL()
String = _String()



# -*- coding: utf-8 -*

from east.asts import ast
from east.asts import utils

class NaiveAnnotatedSuffixTree(ast.AnnotatedSuffixTree):

    __algorithm__ = "ast_naive"

    def _construct(self, strings_collection):
        '''
        Naive generalized suffix tree construction algorithm,
        with quadratic [O(n_1^2 + ... + n_m^2)] worst-case time complexity,
        where m is the number of strings in collection.
        
        '''
        
        # 0. Add a unique character to each string in the collection,
        #    to preserve simplicity while building the tree
        strings_collection = utils.make_unique_endings(strings_collection)
        
        root = ast.AnnotatedSuffixTree.Node()
        root.strings_collection = strings_collection
        
        # For each string in the collection...
        for string_ind in xrange(len(strings_collection)):
            string = strings_collection[string_ind]
            # For each suffix of that string...
            # (do not handle unique last characters as suffixes)
            for suffix_start in xrange(len(string)-1):
                suffix = string[suffix_start:]
                # ... first try to find maximal matching path
                node = root
                child_node = node.chose_arc(suffix)
                while child_node:
                    (str_ind, substr_start, substr_end) = child_node.arc()
                    match = utils.match_strings(
                                suffix, strings_collection[str_ind][substr_start:substr_end])
                    if match == substr_end-substr_start:
                        # matched the arc, proceed with child node
                        suffix = suffix[match:]
                        suffix_start += match
                        node = child_node
                        node.weight += 1
                        child_node = node.chose_arc(suffix)
                    else:
                        # ... then, where the matching path ends;
                        # create new inner node
                        # (that's the only possible alternative
                        #  since we have unique string endings)
                        node.remove_child(child_node)
                        new_node = node.add_new_child(string_ind, suffix_start,
                                                      suffix_start+match)
                        new_leaf = new_node.add_new_child(string_ind, suffix_start+match,
                                                          len(string))
                        (osi, oss, ose) = child_node._arc
                        child_node._arc = (osi, oss+match, ose)
                        new_node.add_child(child_node)
                        new_leaf.weight = 1
                        new_node.weight = 1 + child_node.weight
                        suffix = ''
                        break
                        
                # ... or create new leaf if there was no appropriate arc to proceed
                if suffix:
                    new_leaf = node.add_new_child(string_ind, suffix_start, len(string))
                    new_leaf.weight = 1
                    
        # Root will also be annotated by the weight of its children,
        # to preserve simplicity while calculating string matching
        for k in root.children:
            root.weight += root.children[k].weight
        
        return root



# -*- coding: utf-8 -*

from east import consts


class EastException(Exception):
    """Base EAST Exception.

    To correctly use this class, inherit from it and define
    a 'msg_fmt' property. That msg_fmt will get printf'd
    with the keyword arguments provided to the constructor.
    """
    msg_fmt = "An unknown exception occurred."

    def __init__(self, message=None, **kwargs):
        self.kwargs = kwargs

        if 'code' not in self.kwargs:
            try:
                self.kwargs['code'] = self.code
            except AttributeError:
                pass

        if not message:
            try:
                message = self.msg_fmt % kwargs
            except KeyError:
                exc_info = sys.exc_info()
                # kwargs doesn't match a variable in the message
                # log the issue and the kwargs
                msg = "kwargs don't match in string format operation: %s"
                LOG.debug(msg % kwargs, exc_info=exc_info)

                if CONF.fatal_exception_format_errors:
                    raise exc_info[0], exc_info[1], exc_info[2]
                else:
                    # at least get the core message out if something happened
                    message = self.msg_fmt

        super(EastException, self).__init__(message)

    def format_message(self):
        if self.__class__.__name__.endswith('_Remote'):
            return self.args[0]
        else:
            return unicode(self)


class NotFoundException(EastException):
    msg_fmt = "Not found."


class NoSuchASTAlgorithm(NotFoundException):
    msg_fmt = "There is no AST construction algorithm with name `%(name)s`."


class TomitaNotInstalledException(EastException):
    msg_fmt = ("Please, add the tomita distribution corresponding to your operating system "
               "to `tools/tomita`. The tomita binary file can be downloaded from %s" %
               consts.URL.TOMITA)


class EmptyStringsCollectionException(EastException):
    msg_fmt = "The input strings collection is empty."



# -*- coding: utf-8 -*

from east import consts


def index(array, key, start=0):
    # NOTE(msdubov): No boundary check for optimization purposes.
    i = start
    while array[i] != key:
        i += 1
    return i


def match_strings(str1, str2):
    '''
    Returns the largest index i such that str1[:i] == str2[:i]
    
    '''
    i = 0
    min_len = len(str1) if len(str1) < len(str2) else len(str2)
    while i < min_len and str1[i] == str2[i]: i += 1
    return i


def make_unique_endings(strings_collection):
    '''
    Make each string in the collection end with a unique character.
    Essential for correct builiding of a generalized annotated suffix tree.
    Returns the updated strings collection, encoded in Unicode.

    max strings_collection ~ 1.100.000
    
    '''
    res = []
    for i in range(len(strings_collection)):
        # NOTE(msdubov): a trick to handle 'narrow' python installation issues.
        hex_code = hex(consts.String.UNICODE_SPECIAL_SYMBOLS_START+i)
        hex_code = r"\U" + "0" * (8 - len(hex_code) + 2) + hex_code[2:]
        res.append(strings_collection[i] + hex_code.decode("unicode-escape"))
    return res



# -*- coding: utf-8 -*

from east import utils

def worst_case_strings_collection(m, n):
    # NOTE(msdubov): strings differ only in their last 2 symbols.
    prefix = utils.random_string(n - 2)
    strings_collection = [prefix + utils.random_string(n - 2) for _ in xrange(m)]
    return strings_collection



# -*- coding: utf-8 -*

import gc
import getopt
import os
import psutil
import sys
import time

from analysis import utils
from east.asts import base


def memory_usage():
    # return the memory usage in MB
    process = psutil.Process(os.getpid())
    mem = process.get_memory_info()[0] / float(2 ** 20)
    return mem


def main(args):
    opts, args = getopt.getopt(args, "")

    ast_algorithm = args[0]
    n_from = int(args[1])
    n_to = int(args[2])
    n_step = int(args[3]) if len(args) >= 4 else 1
    m = int(args[4]) if len(args) >= 5 else 100

    repeats = 2  # for each n

    print ast_algorithm
    for n in xrange(n_from, n_to + 1, n_step):
        asts = []
        for _ in xrange(repeats):
            strings_collection = utils.worst_case_strings_collection(m, n)
            ast = base.AST.get_ast(ast_algorithm, strings_collection)
            asts.append(ast)
        print("%i\t%.2f" % (n, memory_usage() / repeats))
        for ast in asts:
            del ast
        gc.collect()
    print ""


if __name__ == '__main__':
    main(sys.argv[1:])
